/**
 * PhoneGap BinaryFileWriter plugin
 *
 * @author Antonio Hernández <ahernandez@emergya.com>
 *
 */

document.addEventListener("deviceready", function() {

    /**
     * Write the contents of a binary array to a file.
     *
     * @param binaryArray	The contents of the file (Array of bytes).
     */
    FileWriter.prototype.writeBinaryArray = function(binaryArray) {

        // Throw an exception if we are already writing a file
        if (this.readyState === FileWriter.WRITING) {
            throw new FileError(FileError.INVALID_STATE_ERR);
        }

        // WRITING state
        this.readyState = FileWriter.WRITING;

        var me = this;

        // If onwritestart callback
        if (typeof me.onwritestart === "function") {
            me.onwritestart(new ProgressEvent("writestart", {"target":me}));
        }

        //cordova.exec()
        //PhoneGap.exec()
        // Write file
        cordova.exec(
            // Success callback
            function(r) {
                // If DONE (cancelled), then don't do anything
                if (me.readyState === FileWriter.DONE) {
                    return;
                }

                // position always increases by bytes written because file would be extended
                me.position += r;
                // The length of the file is now where we are done writing.

                me.length = me.position;

                // DONE state
                me.readyState = FileWriter.DONE;

                // If onwrite callback
                if (typeof me.onwrite === "function") {
                    me.onwrite(new ProgressEvent("write", {"target":me}));
                }

                // If onwriteend callback
                if (typeof me.onwriteend === "function") {
                    me.onwriteend(new ProgressEvent("writeend", {"target":me}));
                }
            },
            // Error callback
            function(e) {
                // If DONE (cancelled), then don't do anything
                if (me.readyState === FileWriter.DONE) {
                    return;
                }

                // DONE state
                me.readyState = FileWriter.DONE;

                // Save error
                me.error = new FileError(e);

                // If onerror callback
                if (typeof me.onerror === "function") {
                    me.onerror(new ProgressEvent("error", {"target":me}));
                }

                // If onwriteend callback
                if (typeof me.onwriteend === "function") {
                    me.onwriteend(new ProgressEvent("writeend", {"target":me}));
                }
            },
            "BinaryFileWriter",
            "writeBinaryArray",
            [this.fileName, binaryArray, this.position]
        );
    };

}, false);